<?php
require_once("3rdparty/domain/IRegistrar.php");
require_once("3rdparty/domain/standardfunctions.php");
/**
 * -------------------------------------------------------------------------------------
 * Versio - IRegistrar
 * 
 * Author		: Versio
 * Copyright	: 2012
 * Version 		: 0.9
 * 
 * CHANGE LOG:
 * -------------------------------------------------------------------------------------
 *  21-05-2012 	Versio 		Initial version
 *  07-06-2012	Versio		Fixed loading contacts bug
 *  08-01-2013	Versio		Checked domain dates
 *  03-03-2013	Versio		Changes to work with newer versions
 *  25-09-2013 	Versio		Bugfixes
 *  
 *  17-08-2015  Versio 		Added DNS support
 *  21-08-2015  Versio 		Fixed duplicate contact bug
 *  18-09-2015  Versio 		Added DNS templates
 *  21-09-2015  Versio 		Added Nameservers support
 *  01-10-2015  Versio   	Added SSL support
 *  01-10-2015  Versio		Added testmode support :)
 *  25-05-2016  Versio		Bugfixes
 * -------------------------------------------------------------------------------------
 */
 
class Versio implements IRegistrar
{
	public $class = 'versio';
    public $User;
    public $Password;
    
    public $Error;
    public $Warning;
    public $Success;
    public $Period = 1;
    public $registrarHandles = array();
    
	public $values = "";
	
    private $ClassName;
    
	function __construct(){	
		
		$this->ClassName = __CLASS__;
		
		$this->Error = array();
		$this->Warning = array();
		$this->Success = array();		
	}
	
	public function versioAPICall($data)
	{   
        $data["klantId"] = $this->User;
		$data["klantPw"] = $this->Password; //SHA1 hash
		$data["sandBox"] = $this->Testmode;

        
		$ch = curl_init("https://www.secure.versio.nl/api/api_server.php");
		curl_setopt($ch, CURLOPT_PORT , 443); 
		curl_setopt($ch, CURLOPT_POST , 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS , $data);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER , 1); 
		curl_setopt($ch, CURLOPT_HTTPHEADER , array('Connection: close'));
		curl_setopt($ch, CURLOPT_TIMEOUT , 120);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); 
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false); 
		$response = curl_exec($ch);
		curl_close($ch);
		
		if($response == "")
		{
			$response = "success=0\nerror=100-000-000\nresponse_message=could not connect with server try again later";
		}
		
		$result = $this->parseResponse($response);
		
		$this->newResponse();
        
        return $result;
	}
	
	public function parseResponse($buffer)
	{
		$lines = explode("\n", $buffer);
		$amount_lines = count($lines);
		
		for($i = 0; $i < $amount_lines; $i++)
		{
			$result = explode("=", $lines[$i]);
			
			if(count($result) >= 2)
			{
				$name = trim($result[0]);
				$value = trim($result[1]);
				$this->values[$name] = $value;
			}
		}
		return  $this->values;
	}
	
	public function newResponse()
	{
		$this->values = "";
	}
	/**
	 * Check whether a domain is already regestered or not. 
	 * 
	 * @param 	string	 $domain	The name of the domain that needs to be checked.
	 * @return 	boolean 			True if free, False if not free, False and $this->Error[] in case of error.
	 */
	function checkDomain($sDomain) 
	{
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$aDomain = explode('.', $sDomain);
		
		$data = array(
			'command' => 'DomainsCheckAvailability',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1]
		);
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		else
		{
			if($result['status'] == 1)
			{
				return true;
			}
			return false;
		}
	}
	
	/**
	 * Register a new domain
	 * 
	 * @param 	string	$domain			The domainname that needs to be registered.
	 * @param 	array	$nameservers	The nameservers for the new domain.
	 * @param 	array	$whois			The customer information for the domain's whois information.
	 * @return 	bool					True on success; False otherwise.
	 */
	function registerDomain($domain, $nameservers = array(), $whois = null) 
	{
		/**
		 * Step 1) obtain an owner handle
		 */
		
		$ownerHandle = "";
		if (isset($whois->ownerRegistrarHandles[$class]))
		{
			$ownerHandle = $whois->ownerRegistrarHandles[$class];
		}
		elseif($whois->ownerSurName != "")
		{	
			$ownerHandle = $this->getContactHandle($whois, HANDLE_OWNER);
			if ($ownerHandle == "") 
			{
				if(!$ownerHandle = $this->createContact($whois, HANDLE_OWNER))
				{
				    return false;
				}			
			}
			$this->registrarHandles['owner'] = $ownerHandle;
		} 
		else 
		{
			$this->Error[] = sprintf("Versio: Er zijn geen houdergegevens opgegeven voor domeinnaam '%s'.", $domain);
			return false;	
		}
		
		/**
		 * COPY HANDLE TO WEFACT 
		 */
		$this->registrarHandles['owner'] = $ownerHandle;
 		
		$this->Period = 1;
		
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$aDomain = explode('.', $domain,2);
		// park?
		$iPark = 1;
		// check
		if(isset($nameservers['ns1']) && !empty($nameservers['ns1']) && $nameservers['ns1'] !== ""){
			$iPark = 0;
			$bNs1 = true;
		}
		if(isset($nameservers['ns2']) && !empty($nameservers['ns2']) && $nameservers['ns2'] !== ""){
			$iPark = 0;
			$bNs2 = true;
		}
		if(isset($nameservers['ns3']) && !empty($nameservers['ns3']) && $nameservers['ns3'] !== ""){
			$iPark = 0;
			$bNs3 = true;
		}
		$data = array(
			'command' => 'DomainsRegister',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1],
			'years' => '1',
			'contact_id' => $ownerHandle['id'],
			'park' => $iPark,
		);
		if(isset($bNs1))
			$data['ns1'] = $nameservers['ns1'];
		if(isset($bNs2))
			$data['ns2'] = $nameservers['ns2'];
		if(isset($bNs3))
			$data['ns3'] = $nameservers['ns3'];
		
		$result = $this->versioAPICall($data);

		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: foutcode ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		else
		{
 			return true;
		}
	}
	
	/**
	 * Transfer a domain to the given user.
	 * 
	 * @param 	string 	$domain			The demainname that needs to be transfered.
	 * @param 	array	$nameservers	The nameservers for the tranfered domain.
	 * @param 	array	$whois			The contact information for the new owner, admin, tech and billing contact.
	 * @return 	bool					True on success; False otherwise;
	 */
	function transferDomain($domain, $nameservers = array(), $whois = null, $authcode = "") {
		$ownerHandle = "";
		if (isset($whois->ownerRegistrarHandles[$class]))
		{
			$ownerHandle = $whois->ownerRegistrarHandles[$class];
		}
		elseif($whois->ownerSurName != "")
		{
			$ownerHandle = $this->getContactHandle($whois, HANDLE_OWNER);
			if ($ownerHandle == "") 
			{
				if(!$ownerHandle = $this->createContact($whois, HANDLE_OWNER))
				{
				    return false;
				}			
			}
			$this->registrarHandles['owner'] = $ownerHandle;
		} else {
			$this->Error[] = sprintf("Versio: Er zijn geen houdergegevens opgegeven voor domeinnaam '%s'.", $domain);
			return false;	
		}
		
		/**
		 * COPY HANDLE TO WEFACT 
		 */
		$this->registrarHandles['owner'] = $ownerHandle;
		
 		$this->Period = 1;
 		
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$aDomain = explode('.', $domain);
		
		
		// park?
		$iPark = 1;
		// check
		if(isset($nameservers['ns1']) && !empty($nameservers['ns1']) && $nameservers['ns1'] !== ""){
			$iPark = 0;
			$bNs1 = true;
		}
		if(isset($nameservers['ns2']) && !empty($nameservers['ns2']) && $nameservers['ns2'] !== ""){
			$iPark = 0;
			$bNs2 = true;
		}
		if(isset($nameservers['ns3']) && !empty($nameservers['ns3']) && $nameservers['ns3'] !== ""){
			$iPark = 0;
			$bNs3 = true;
		}
		
		$data = array(
			'command' => 'DomainsTransfer',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1],
			'code' => $authcode,
			'years' => '1',
			'contact_id' => $ownerHandle,
			'park' => $iPark
		);
		
		if(isset($bNs1))
			$data['ns1'] = $nameservers['ns1'];
		if(isset($bNs2))
			$data['ns2'] = $nameservers['ns2'];
		if(isset($bNs3))
			$data['ns3'] = $nameservers['ns3'];
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: foutcode ' . $result['command_response_code'].' - ' . $result['command_response_message'] . var_export($result, true);
 			return false;
		}
		elseif($result['success'] == 1)
		{
 			return true;
		}
		else
		{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}
	
	/**
	 * Extend a domain for several years
	 * This function is currently not used in WeFact Hosting
	 * 
	 * @param 	string 	$domain		The name of the domain you want to extend.
	 * @param 	int		$nyears		The number of year the domain should be extended.
	 * @return	bool				True if the domain was extended succesfully; False otherwise;
	 */
	function extendDomain($domain, $nyears) {
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$aDomain = explode('.', $domain);
		$data = array(
			'command' => 'DomainsRenew',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1],
			'years' => $nyears
		);
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		elseif($result['success'] == 1)
		{
 			return true;
		}
		else
		{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}
	
	/**
	 * Delete a domain
	 * 
	 * @param 	string $domain		The name of the domain that you want to delete.
	 * @param 	string $delType     end|now
	 * @return	bool				True if the domain was succesfully removed; False otherwise; 
	 */
	function deleteDomain($domain, $delType = 'end') {
		return $this->setDomainAutoRenew($domain, false);
	}
	
	/**
	 * Get all available information of the given domain
	 * 
	 * @param 	mixed 	$domain		The domain for which the information is requested.
	 * @return	array				The array containing all information about the given domain
	 */
	function getDomainInformation($domain) {
     	
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$aDomain = explode('.', $domain);
		
		$data = array(
			'command' => 'DomainsListSingle',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1]
		);
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: foutcode ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		elseif($result['success'] == 1)
		{
			if($result['status'] == 'Actief')
			{
				$nameservers = array(
					$result['ns1'],
					$result['ns2'],
					$result['ns3']
				);
				
				$aDateParts = explode('-', $result['expiration_date']);
				
				//$expirationDate = rewrite_date_db2site($result['expiration_date']);
				$expirationDate = rewrite_date_db2site($aDateParts[2].'-'.$aDateParts[1].'-'.$aDateParts[0]);
				
				/**
				 * Versio API ondersteunt geen registratie datum (niet op te vragen) en / of contact id (whois handle).
				 */
				$registrationDate = 'Unknown';
				
				$whois = new whois();
				$whois->ownerHandle = 'ABCD001';
				$whois->adminHandle = 'ABCD001';
				$whois->techHandle 	= 'ABCD002';
				
				//$this->Error[] 	= 'Versio: Momenteel is het ophalen van contacten nog niet mogelijk.';
				
				$response = array(
					'Domain' => $domain,
					'Information' => array(
						'nameservers' => $nameservers,
						'whois' => $whois,
						'expires' => $expirationDate,
						'regdate' => $registrationDate,
						'authkey' => $this->getToken($domain)
					)
				);
				return $response;
			}
			else
			{
				$this->Error[] = 'Versio: De domeinnaam ' . $domain . ' is niet meer actief.';
				return false;
			}
		}
		else
		{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}
	
	/**
	 * Get a list of all the domains.
	 * 
	 * @param 	string 	$contactHandle		The handle of a contact, so the list could be filtered (usefull for updating domain whois data)
	 * @return	array						A list of all domains available in the system.
	 */
	function getDomainList($contactHandle = "") {
		

		// $this->Error[] = 'Versio: Deze functie wordt momenteel niet door versio ondersteund.';
		// return false;
		
		if($contactHandle != "")
		{
			$this->Error[] = 'Versio: Het filteren op een contact is momenteel niet mogelijk via de Versio API.';
		}

		
		# create api command
		$data = array(
			'command' => 'DomainsListActive'
		);

		# execute command
		$result = $this->versioAPICall($data);

		# no result?
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
			return false;
		}

		# otherwise
		else
		{	
			# retrieve contacts list
			$handles = $this->getContactList();

			# create handle list to store the result in
			$handle_list = array();

			# loop through the handles
			foreach ($handles as $handle)
			
				$handle_list[$handle['Handle']] = $handle;
			

			if($result['total_count'] > 0)
			{

				$i = 1;
				$domainsList = array();
				while($result['total_count'] >= $i)
				{

					$nameservers = array(
						$result['ns1_' . $i],
						$result['ns2_' . $i],
						$result['ns3_' . $i]
					);

					
					//$expirationDate = rewrite_date_db2site($result['expiration_date_' . $i]);
					$aDateParts = explode('-', $result['expiration_date_'.$i]);
					//$expirationDate = rewrite_date_db2site($result['expiration_date']);
					$expirationDate = rewrite_date_db2site($aDateParts[2].'-'.$aDateParts[1].'-'.$aDateParts[0]);
					
				    /**
					* TODO - Versio API (REG DATE?)
					* TODO - Whois contact (get contact id from versio?) -- NIET MOGELIJK OP DIT MOMENT
					*/
$registrationDate = 'Unknown';
$whois = new Whois();

$contactid = $result['contactid_'.$i];

$whois->ownerHandle = $contactid;

if($contactid == '0')
{
$whois->ownerHandle = $handles[0]['Handle'];
$whois->ownerSex = $handles[0]['Sex'];
$whois->ownerInitials = $handles[0]['Initials'];
$whois->ownerSurName = $handles[0]['SurName'];
$whois->ownerCompanyName = $handles[0]['CompanyName'];

$whois->ownerAddress = $handles[0]['Address'];
$whois->ownerZipCode = $handles[0]['ZipCode'];
$whois->ownerCity = $handles[0]['City'];
$whois->ownerCountry = $handles[0]['Country'];

$whois->ownerPhoneNumber = $handles[0]['PhoneNumber'];
$whois->ownerEmailAddress = $handles[0]['EmailAddress'];
}

$whois->adminHandle = 'ABCD001';
$whois->techHandle = 'ABCD001';

//var_dump($whois);
			
					$domain = array();
					$domain['Domain'] = $result['domain_' . $i] . '.' . $result['tld_' . $i];
					$domain['Information'] = array(
						'nameservers' => $nameservers,
						'whois' => $whois,
						'expires' => $expirationDate,
						'regdate' => $registrationDate,
						'authkey' => $this->getToken($domain)
					);

					// push each result to the end of the array
					array_push($domainsList, $domain);
					$i++;
				}
				# return the result
				return $domainsList;

			}
		}
	}
	
	/**
	 * Change the lock status of the specified domain.
	 * 
	 * @param 	string 	$domain		The domain to change the lock state for
	 * @param 	bool 	$lock		The new lock state (True|False)
	 * @return	bool				True is the lock state was changed succesfully
	 */
	function lockDomain($domain, $lock = true) {

		$aDomain = explode('.', $domain,2);

		# lock or unlock?
		$bLock = ($lock == true) ? 1 : 0;
	
		unset($command);
		$command = array(
			"command" => "DomainsSetLock",
			"domain" => $aDomain[0],
			"tld" => $aDomain[1],
			"lock" => $bLock
		);

		$result = $this->versioAPICall($command);

		if($result['success']==0)
			$this->Error[] = "Fout opgetreden. Fout code: ".$result['command_response_code'].". Fout text: ".$result['command_response_message'];
		else
			return true;
	}
	
	/**
	 * Change the autorenew state of the given domain. When autorenew is enabled, the domain will be extended.
	 * 
	 * @param 	string	$domain			The domainname to change the autorenew setting for,
	 * @param 	bool	$autorenew		The new autorenew setting (True = On|False = Off)
	 * @return	bool					True when the setting is succesfully changed; False otherwise
	 */
	function setDomainAutoRenew($domain, $autorenew = true) {
		
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$autoRenew = ($autorenew == true) ? '1' : '0';
		$aDomain = explode('.', $domain);
		
		$data = array(
			'command' => 'DomainsSetAutoRenew',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1],
			'auto_renew' => $autoRenew
		);
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: foutcode ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		elseif($result['success'] == 1)
 			return true;
		
		# catch all else
		else{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}
	
	/**
	 * Update the domain Whois data, but only if no handles are used by the registrar.
	 * 
	 * @param mixed $domain
	 * @param mixed $whois
	 * @return boolean True if succesfull, false otherwise
	 */
	function updateDomainWhois($domain,$whois){
		$aDomain = explode('.', $domain);
		if($aDomain[1] == 'be')
		{
			$this->Error[] = 'Versio: Voor een .be domeinnaam moet de EPP code eerst ingevuld zijn. Deze functie wordt momenteel nog niet ondersteund.';
			return false;
		}
		
		$contactId = $this->createContact($whois);
		
		$data = array(
			'command' => 'DomainsChangeOwner',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1],
			'newcontact_id' => $contactId
		);
		
		if($aDomain[1] == 'be')
			$data['eppcode'] = $this->getToken($domain); 
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		elseif($result['success'] == 1)
		{
			$this->Success[] = 'Versio: Domeinnaam houder succesvol gewijzigd';
 			return true;
		}
		else
		{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}
	
	/**
	 * get domain whois handles
	 * 
	 * @param mixed $domain
	 * @return array with handles
	 */
	function getDomainWhois($domain){
		
		$contacts = array();
		$contacts['ownerHandle'] 	= 'ABC001';
		$contacts['adminHandle'] 	= $contacts['ownerHandle'];
		$contacts['techHandle'] 	= $contacts['ownerHandle'];
		
		return $contacts;
	}
	/**
	 * Get EPP code/token
	 * 
	 * @param mixed $domain
	 * @return 
	 */
	public function getToken($domain)
	{				


		# explode the domain name
		$aDomain = explode('.', $domain);
		
		if($aDomain[1] == 'be')
		{
		return 'Request EPP code on DNS.be';
		# try the other function if failed return error
		}else{
		$data = array(
				'command' => 'DomainsGetEPP',
				'domain' => $aDomain[0],
				'tld' => $aDomain[1]
			);
			$result = $this->versioAPICall($data);
			if($result['success'] == 0)
			{
				//$this->Error[] = 'Versio: Domain ' . $aDomain[0].'.'.$aDomain[1] . ' not supported via our API.';
				return false;
			}
			else
			{
				return $result['token'];
			}
		}
	}
	
	/**
	 * Create a new whois contact
	 * 
	 * @param 	array		 $whois		The whois information for the new contact.
	 * @param 	mixed 	 	 $type		The contact type. This is only used to access the right data in the $whois object.
	 * @return	bool					Handle when the new contact was created succesfully; False otherwise.		
	 */
	function createContact($whois, $type = HANDLE_OWNER) {
		// Determine which contact type should be found
		switch($type) {
			case HANDLE_OWNER:	$prefix = "owner"; 	break;	
			case HANDLE_ADMIN:	$prefix = "admin"; 	break;	
			case HANDLE_TECH:	$prefix = "tech"; 	break;	
			default:			$prefix = ""; 		break;	
			
		}
		$whois->getParam($prefix,'Address');
		
		$countryCode = $whois->{$prefix.'Country'};
		
		if(strlen($countryCode) > 2)
		{
			$countryCode = str_replace('EU-', '', $countryCode);
		}
		
		$data = array(
			'command' => 'DomainsListCountries'
		);
		
		$result = $this->versioAPICall($data);
					
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
			return false;
		}
		else
		{
			if($result['total_count'] > 0)
			{
				for($i=0; $i<= $result['total_count']; $i++)
				{
					if($countryCode == $result['landcode' . $i])
					{
						$countryCode = $result['landcode' . $i];
						break;
					}
				}
			}
		}
		
		if(strlen($countryCode) != 2)
		{
			$countryCode = 'NL';
		}
		
		/*$zipCode = $whois->{$prefix.'ZipCode'};
		if(strlen($zipCode) != 7)
		{
			$digits = substr($zipCode, 0, 4);
			$letters = substr($zipCode, 5, 6);
			$zipCode = $digits . ' ' . $letters;
		}*/
		
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */	
		
		$sStreet = $whois->{$prefix.'Address'};
		$iNumber = filter_var($sStreet, FILTER_SANITIZE_NUMBER_INT);
		$sStreet = str_replace($iNumber, '', $sStreet);
		$data = array(
			'command' => 'DomainsCreateContact',
			'companyname' => $whois->{$prefix.'CompanyName'},
			'initials' => $whois->{$prefix.'Initials'},
			'surname' => $whois->{$prefix.'SurName'},
			'email' => $whois->{$prefix.'EmailAddress'},
			'phone' => $whois->{$prefix.'PhoneNumber'},
			'street' => $sStreet,
			'hnr' => $iNumber,
			'hnradd' => $whois->{$prefix.'StreetNumberAddon'},
			'zipcode' => $whois->{$prefix.'ZipCode'},
			'city' => $whois->{$prefix.'City'},
			'country' => $countryCode
		);
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		else
		{
			/**
			 * RETURN CONTACT_ID
			 */
 			return $result['contact_id'];
		}
	}
	
	/**
	 * Update the whois information for the given contact person.
	 * 
	 * @param string $handle	The handle of the contact to be changed.
	 * @param array $whois The new whois information for the given contact.
	 * @param mixed $type The of contact. This is used to access the right fields in the whois array
	 * @return
	 */
	function updateContact($handle, $whois, $type = HANDLE_OWNER) {
		// Determine which contact type should be found
		switch($type) {
			case HANDLE_OWNER:	$prefix = "owner"; 	break;	
			case HANDLE_ADMIN:	$prefix = "admin"; 	break;	
			case HANDLE_TECH:	$prefix = "tech"; 	break;	
			default:			$prefix = ""; 		break;		
		}

		
		$this->Error[] = sprintf("Versio: Het bewerken van een contact is niet mogelijk.");
		return false;
		

		# first delete the old owner information
		$this->deleteContact($handle);
		
		# regex to split housenumber and address from each other
		$iHouseNumber = preg_replace( '/[^0-9]/', '', $whois->{$prefix.'Address'});
		$sHouseAddr	  = preg_replace( '/([^a-zA-Z])/', '', $whois->{$prefix.'Address'});

		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */		
		$data = array(
			'command' => 'DomainsCreateContact',
			'companyname' => $whois->{$prefix.'CompanyName'},
			'initials' => $whois->{$prefix.'Initials'},
			'surname' => $whois->{$prefix.'SurName'},
			'email' => $whois->{$prefix.'EmailAddress'},
			'phone' => $whois->{$prefix.'PhoneNumber'},
			'street' => $sHouseAddr,
			'hnr' => $iHouseNumber,
			'hnradd' => $whois->{$prefix.'StreetNumberAddon'},
			'zipcode' => $whois->{$prefix.'ZipCode'},
			'city' => $whois->{$prefix.'City'},
			'country' => $whois->{$prefix.'Country'}
		);

		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		elseif($result['success'] == 1)
		{
 			return $result['contact_id'];
		}
		else
		{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}

	/**
	 * delete a contact
	 *
	 * @param(array) information availabe about the requested contact.
	 * @return bool
	 */
	function deleteContact ($handle)
	{
		$command = array(
			'command' => 'DomainsDeleteContact',
			'id' => $handle
		);

		# execute the command and store the result
		$result = $this->versioAPICall($command);
		if($result['success'] == 0)
			$this->Error[] = 'Versio: foutcode ' . $result['command_response_code'].' - ' . $result['command_response_message'];
		else
			return true;

		return false;
	}


	
	/**
     * Get information availabe of the requested contact.
     * 
     * @param string $handle The handle of the contact to request.
     * @return array Information available about the requested contact.
     */
    function getContact($handle)
	{        
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$data = array(
			'command' => 'DomainsListSingleContact',
			'id' => $handle
		);
		$result = $this->versioAPICall($data);
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: foutcode ' . $result['command_response_code'].' - ' . $result['command_response_message'];
			return false;
		}
		else
		{
			$response = true;
		}
		$whois 		= new whois();
			
		/**
		 * Step 2) provide feedback to WeFact
		 */
		if($response)
		{
			// The contact is found
			$whois->ownerCompanyName 	= $result['companyname'];
			$whois->ownerInitials		= $result['initials'];
			$whois->ownerSurName		= $result['surname'];
			$whois->ownerAddress 		= $result['street'];
			$whois->ownerZipCode 		= $result['zipcode'];
			$whois->ownerCity 			= $result['city'];
			
			$country = $result['companyname'];
			global $array_country;
			
			$whois->ownerCountry 		= (in_array($country, $array_country)) ? $country : ((in_array("EU-".$country, $array_country)) ? "EU-".$country : '');
			
			$whois->ownerPhoneNumber	= $result['phone'];
			$whois->ownerFaxNumber 		= '';   
			$whois->ownerEmailAddress	= $result['email'];
			
			return $whois;
		}
		else
		{
			// Contact cannot be found
			$this->Error[] 	= sprintf("Versio: Fout tijdens ophalen contact: %s", $error_msg);
			return $whois;			
		}
	}
		
	
	/**
     * Get the handle of a contact.
     * 
     * @param array $whois The whois information of contact
     * @param string $type The type of person. This is used to access the right fields in the whois object.
     * @return string handle of the requested contact; False if the contact could not be found.
     */
    function getContactHandle($whois = array(), $type = HANDLE_OWNER) {
			
		// Determine which contact type should be found
		switch($type) {
			case HANDLE_OWNER:  $prefix = "owner";  break;	
			case HANDLE_ADMIN:  $prefix = "admin";  break;	
			case HANDLE_TECH:   $prefix = "tech";   break;	
			default:            $prefix = "";       break;	
		}

		/**
		 * Step 1) Search for contact data
		 */
		// Search for a handle which can be used
		unset($handle);
		unset($command);

		$command = array(
			"command" => "DomainsListSingleContact",
			"id" => $whois->ownerRegistrarHandles['versio']
		);

		$result = $this->versioAPICall($command);
		
		/**
		 * Step 2) provide feedback to WeFact
		 */

		if($result['success'] == 0)
		{
			// No handle is found
			return false;
		}
		else
		{
			return $result;	
		}

		return true;
   	}
	
	/**
     * Get a list of contact handles available
     * 
     * @param string $surname Surname to limit the number of records in the list.
     * @return array List of all contact matching the $surname search criteria.
     */
    function getContactList($surname = "") {
		
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$data = array(
			'command' => 'DomainsListContacts'
		);
		$result = $this->versioAPICall($data);
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
			return false;
		}
		else
		{
			$contactsList = array();
			if($result['total_count'] > 0)
			{
				$i = 1;
while($result['total_count'] >= $i)
{
$contact = array();
$contact['Handle'] = $result['id_' . $i];
$contact['Sex'] = 'm'; // Not provided in Versio API
$contact['Initials'] = $result['initials_' . $i];
$contact['SurName'] = $result['surname_' . $i];
$contact['CompanyName'] = $result['companyname_' . $i];

$contact['Address'] = trim($result['street_' . $i] . " " . $result['hnr_' . $i]);
$contact['ZipCode'] = $result['zipcode_' . $i];
$contact['City'] = $result['city_' . $i];
$contact['Country'] = strtoupper($result['country_' . $i]);

$contact['PhoneNumber'] = $result['phone_' . $i];
$contact['EmailAddress'] = $result['email_' . $i];

$contactsList[] = $contact;
$i++;
}
			}
			return $contactsList;
		}
   	}
	/**
   	 * Update the nameservers for the given domain.
   	 * 
   	 * @param string $domain The domain to be changed.
   	 * @param array $nameservers The new set of nameservers.
   	 * @return bool True if the update was succesfull; False otherwise;
   	 */
   	function updateNameServers($domain, $nameservers = array()) {
		
		/**
		 * MAKE THE VERSIO API CALL AND RETURN RESPONSE TO WEFACT
		 */
		$aDomain = explode('.', $domain);
		# Check
		if($nameservers['ns1'] == '' && $nameservers['ns2'] == '' && $nameservers['ns3'] == ''){
			
			$this->Error[] = 'U heeft geen nameservers ingevoerd.';
			return false;
			
		}
		$data = array(
			'command' => 'DomainsSetNS',
			'domain' => $aDomain[0],
			'tld' => $aDomain[1],
			'ns1' => $nameservers['ns1'],
			'ns2' => $nameservers['ns2'],
			'ns3' => $nameservers['ns3']
		);
		
		$result = $this->versioAPICall($data);
		
		if($result['success'] == 0)
		{
			$this->Error[] = 'Versio: Code: ' . $result['command_response_code'].' - ' . $result['command_response_message'];
 			return false;
		}
		elseif($result['success'] == 1)
		{
 			return true;
		}
		else
		{
			$this->Error[] = 'Versio: Onbekende fout.';
			return false;
		}
	}
	
	/**
	 * Optional function to check the status of a registration or transfer, see appendix B of documentation
	 * 
	 * @param mixed $domain
	 * @param mixed $pendingInfo
	 * @return
	 */
	function doPending($domain, $pendingInfo){
		switch($pendingInfo['StatusCode']){
			case 'RegStarted': 
				// Is registration finished succesfully?
				
				// Check current status
				$result = true;
				$error  = false;
				
				if($result){
					$this->Success[] = "Domeinnaam '".$domain."' is succesvol aangevraagd.";
					return true;
				}elseif($error){
					$this->Error[] = "Domeinnaam '".$domain."' is niet succesvol aangevraagd: ".$error;
					return false;
				}
				break;
		}
		
		// Stil pending...
		return 'pending';
	}
	
	/**
	 * Get class version information.
	 * 
	 * @return array()
	 */
	static function getVersionInformation() {
		require_once("3rdparty/domain/versio/version.php");
		return $version;	
	}


	/**
	 * Get the DNS zone of a domain
	 *
	 * @param(String) the domain
	 * @return(Array)
	 */
	function getDNSZone($domain)
	{	

		$domainExploded = explode('.', $domain,2);

		# the command
		$command = array(
			"command" => "DomainsDNSListRecords",
			"domain" => $domainExploded[0],
			"tld" => $domainExploded[1]
		);

		# execute the command
		$result = $this->versioAPICall($command);

		# add the error if we have one
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];

		else{

			# do we have a result?
			if($result['total_count']>0){

				$i = 0;
				$teller = 1; // set-up
				// loop them all
				while($result['total_count']>=$teller) {
					$record_type = 'records';
					$dns_zone[$record_type][$i]['name']     = $result['name_'.$teller];
			        $dns_zone[$record_type][$i]['type']     = $result['type_'.$teller];
			        $dns_zone[$record_type][$i]['value']    = $result['value_'.$teller];
			        $dns_zone[$record_type][$i]['priority'] = $result['prio_'.$teller];
			        $dns_zone[$record_type][$i]['ttl']      = $result['ttl_'.$teller];
            		$dns_zone[$record_type][$i]['id']       = $result['id_'.$teller]; // not required but we provide
					// increment
					$i++;
					$teller++;
				}

				# geef waarde terug
				return $dns_zone;
			}else
				$this->Error[] = 'Er zijn geen DNS records gevonden voor deze domein.';
		}

		# return false
		return false;
	}

	/**
	 * Get all the dns templates
	 * @return(array) or throw false
	 */
	function getDNSTemplates(){

		# the command
		$command = array(
			'command' => 'DNSTemplatesList'
		);
		
		# execute and store the result
		$result = $this->versioAPICall($command);

		# failed to execute? output error otherwise procceed
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];

		if($result['total_count'] > 0){

			# create array
			$dns_templates = [];

			# initialize
			$teller = 1;
			while($result['total_count']>= $teller) {
					
				# store result in array
				$dns_templates['templates'][$teller]['id'] = $result['id_'.$teller];
            	$dns_templates['templates'][$teller]['name'] = $result['name_'.$teller];;

            	# increment
				$teller++;
			}

			# return result and exit function
			return $dns_templates;
		}


		# not found
		$this->Error[] = 'Er zijn geen DNS templates in uw account';
		return false;
	}

	/**
	 * save a dns zone
	 * @param(string) the domain
	 * @param(string) the dns zone
	 */
	function saveDNSZone($domain, $dns_zone) {

		# get the old zone first
		$old_dns_zone = $this->getDNSZone($domain);

		# remove all old settings
		foreach($old_dns_zone['records'] as $record)
			$this->deleteDnsRecord($domain, $record['id']);
		
		# set the new settings
		foreach($dns_zone['records'] as $dns)
			$this->addDnsRecord($domain, $dns);
		
		# return true
		return true;
	}

	function addDnsRecord($domain, $dns){

		# explode domain
		$domainExploded = explode('.', $domain,2);

		# unset and initilize command
		unset($command);
			$command = array(
			"command" => "DomainsDNSAddRecord",
			"domain" => $domainExploded[0],
			"tld" => $domainExploded[1],
			"name" => $dns['name'],
			"type" => $dns['type'],
			"value" => $dns['value'],
			"prio" => $dns['priority'],
			"ttl" => $dns['ttl']
		);
		
		# excecution		
		$result = $this->versioAPICall($command);

		# provide feedback
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else
			return true;

		# return false
		return false;
	}

	/**
	 * @param(domain)
	 */
	function deleteDnsRecord($domain, $zoneId){

		# explode domain
		$domainExploded = explode('.', $domain,2);

		# unset and initialize command
		unset($command);
		$command = array(
			"command" => "DomainsDNSDeleteRecord",
			"domain" => $domainExploded[0],
			"tld" => $domainExploded[1],
			"id" => $zoneId
		);
		
		# excecution
		$result = $this->versioAPICall($command);
		
		# provide feedback
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else
			return true;

		# return false
		return false;
	}





	 /**
     * Retrieve all SSL products from registrar
     *
     * @param string $ssl_type  Optional filter for SSL validation type
     * @return array
     */
    public function ssl_list_products($ssl_type = '')
    {	
    	# setup
        unset($command);
		$command = array(
			"command" => "SSLListProducts"
		);

		# empty array
		$products_array = array();

		# execute
		$result = $this->versioAPICall($command);

		# provide feedback
		if($result['success']==0) {
			$this->Error[] = "Fout opgetreden. Fout code: ".$result['command_response_code'].". Fout text: ".$result['command_response_message'];
		}
		else{
			
			if($result['total_count']>0){
			
			$teller = 1;
			while($result['total_count']>=$teller) {

				$product['name'] 			= $result['supplier_'.$teller] .' ('.$result['type_'.$teller] .'), '. '(' . $result['price_1year_'.$teller] . ')';
				$product['brand'] 			= $result['supplier_'.$teller];
				$product['templatename'] 	= $result['id_'.$teller];
				$product['type'] 			= $result['type_'.$teller];
				$product['wildcard'] 		= $result['multi_domain_'.$teller];
				$product['multidomain_max']	= $result['multi_domain_'.$teller];
				$product['max_years']		= $result['max_years_'.$teller];
				$product['pricing']['period'] = array(
													1 => $result['price_1year_'.$teller],
													2 => $result['price_2year_'.$teller],
													3 => $result['price_3year_'.$teller]
												);

				# push it
				array_push($products_array, $product);
				
				$teller++;
			}
		}
		
		else
			$this->Error[] = "Er zijn geen producten gevonden.";
			
		}
        return $products_array;
    }

    /**
     * Get SSL product from registrar
     *
     * @param string $templatename     ID of the product as retrieved by ssl_list_products()
     * @return array
     */
    public function ssl_get_product($templatename)
    {
        foreach ($this->ssl_list_products() as $sslProductList) {
        	
        	if($sslProductList['templatename'] == $templatename){
        		$sslProduct = $sslProductList;
        	}
        }


        $product_info = array();

     
            $product_info['name'] 			= $sslProduct['name'];
            $product_info['brand']          = $sslProduct['brand'];
            $product_info['templatename'] 	= $sslProduct['templatename'];
            $product_info['type'] 			= $sslProduct['type']; // domain, extended or organization

            $product_info['wildcard']		        = ($sslProduct['wildcard'] != NULL) ? TRUE : FALSE;
            $product_info['multidomain']			= ($sslProduct['multidomain_max'] != NULL) ? TRUE : FALSE;
            // $product_info['multidomain_included']	= ($sslProduct['domains']) ? $sslProduct['domains'] : 0;
            if($product_info['multidomain_max'])
            	$product_info['multidomain_max']	= 99;

            // Pricing-periods
            $product_info['periods'] = array();

            if(isset($sslProduct['pricing']))
            {
                foreach($sslProduct['pricing']['period'] as $index => $period_fee)
                {
                    // please note: periods should be in years, not months
                    $product_info['periods'][] = array('periods' => (int) $period_fee['years'], 'price' => (float) $period_fee['price']);
                }
            }
       
        return $product_info;
    }

    /**
     * Request the SSL certificate at the registrar
     *
     * @param array $ssl_info   Array with info regarding the SSL certificate
     * @param object $whois     Object with WHOIS data
     * @return bool
     */
    public function ssl_request_certificate($ssl_info, $whois)
    {
        unset($command);
		$command = array(
			"command" => "SSLCertificateOrder",
			"product_id" => $ssl_info['templatename'],
			"csr" => $ssl_info['csr'],
			"term" => $ssl_info['period'],
			"approver_email" => $ssl_info['approver_email'],
			"street" => $whois->ownerAddress,
			"zipcode" => $whois->ownerZipCode,
			"san_names" => $ssl_info['multidomain_records']
		);

		$result = $this->versioAPICall($command);

		# provide feedback
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else{
			return $result['id'];
		}

		return false;
    }

    /**
     * Retrieve the approver emailadresses for a SSL certificate from the registrar
     *
     * @param string $commonname    Domain name
     * @param string $templatename  ID of the product as retrieved by ssl_list_products()
     * @return array|bool
     */
    public function ssl_get_approver_list($commonname, $templatename)
    {
		$command = array(
			"command" => "SSLListApproverEmails",
			"product_id" => $templatename,
			"domain" => $commonname
		);

		// execute
		$result = $this->versioAPICall($command);

		// provide feedback
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		
		// record found fetch it
		else{
			$teller = 1;
			while($result['total_count']>= $teller) {
					$approverEmails[] = $result['email_' . $teller];
					$teller++;
			}
			return $approverEmails;
		}
		return false;
    }

    /**
     * Retrieve the status of a SSL certificate from the registrar
     *
     * @param string $ssl_order_id  Order id from a SSL certificate
     * @return array
     */
    public function ssl_get_request_status($ssl_order_id)
    {
        /**
         * Step 1) get status of SSL certificate
         */
        unset($command);
        $command = array(
			"command" => "SSLListSingleCertificate",
			"id" => $ssl_order_id
		);
        
        // execute
        $result = $this->versioAPICall($command);

        // provide feedback
		if($result['success'] == 0){
			$order_info['status'] = 'error';
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		}else{

			$order_info = array();

			switch (strtolower($result['issued'])) {
				case 0:
					$order_info['status'] = 'inrequest';
					break;
				case 1:
					$order_info['status'] = 'install';
					break;
				
				default:
					$order_info['status'] = 'error'; 
					break;
			}
			// return order information
			return $order_info;
		}
		return false;
	}

    /**
     * Download the CSR of a SSL certificate from the registrar
     *
     * @param string $ssl_order_id  Order id from a SSL certificate
     * @return bool|string
     */
    public function ssl_download_ssl_certificate($ssl_order_id)
    {
        unset($command);
		$command = array(
			"command" => "SSLListSingleCertificate",
			"id" => $ssl_order_id
		);

		// execute
        $result = $this->versioAPICall($command);

        if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else{
			$certificate = base64_decode($result['certificate']);

			return $certificate;
		}
		return false;
    }

    /**
     * Reissue a SSL certificate at the registrar
     *
     * @param string $ssl_order_id  Order id from a SSL certificate
     * @param array $ssl_info       Array with info regarding the SSL certificate
     * @param object $whois         Object with WHOIS data
     * @return bool
     */
    public function ssl_reissue_certificate($ssl_order_id, $ssl_info, $whois)
    {
        unset($command);
		$command = array(
			"command" => "SSLCertificateReissue",
			"id" => $ssl_order_id,
			"csr" => $ssl_info['csr'],
			"approver_email" => $ssl_info['approver_email'],
			"street" => $whois->ownerAddress,
			"zipcode" => $whois->ownerZipCode
		);

		// execute
        $result = $this->versioAPICall($command);

        if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else
			return true;

		return false;
    }

    /**
     * Renew the SSL certificate at the registrar
     *
     * @param array $ssl_info       Array with info regarding the SSL certificate
     * @param object $whois         Object with WHOIS data
     * @return bool
     */
    public function ssl_renew_certificate($ssl_info, $whois)
    {
        unset($command);
		$command = array(
			"command" => "SSLCertificateRenew",
			"id" => $ssl_info['templatename'],
			"term" => $ssl_info['period'],
			"csr" => $ssl_info['csr'],
			"approver_email" => $ssl_info['approver_email'],
			"street" => $whois->ownerZipCode,
			"zipcode" => $whois->ownerCity
		);


		// execute
        $result = $this->versioAPICall($command);

        // feedback
        if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else
			return $result['id'];

		return false;
    }

    /**
     * Resend the approver email
     *
     * @param string $ssl_order_id      Order id from a SSL certificate
     * @param $approver_emailaddress    Approver emailaddress
     * @return bool
     */
    public function ssl_resend_approver_email($ssl_order_id, $approver_emailaddress)
    {
        unset($command);
			$command = array(
			"command" => "SSLChangeApprover",
			"id" => $ssl_order_id,
			"approver_email" => $approver_emailaddress
		);

		// execute
		$result = $this->versioAPICall($command);

		// result
		if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else
			return true;
		
		return false;
    }

    /**
     * Revoke SSL certificate at the registrar
     *
     * @param string $ssl_order_id      Order id from a SSL certificate
     * @return bool
     */
    public function ssl_revoke_ssl_certificate($ssl_order_id)
    {
        /**
         * Step 1) revoke the SSL certificate at the registrar
         */
        unset($command);
		$command = array(
			"command" => "SSLCertificateCancelValidation",
			"id" => $ssl_order_id
		);
        
        // execute
		$result = $this->versioAPICall($command);

        // result
        if($result['success'] == 0)
			$this->Error[] = 'Fout opgetreden. Fout code: ' . $result['command_response_code'] . '. Fout text: '. $result['command_response_message'];
		else
			return true;
		
		return false;
    }
}
